# -*- coding: utf-8 -*-
from amplify.agent.context import context
from amplify.agent.containers.nginx.config.config import NginxConfig
from amplify.agent.containers.abstract import AbstractCollector
from amplify.agent.util.http import HTTPClient
from amplify.agent.eventd import CRITICAL, WARNING

__author__ = "Mike Belov"
__copyright__ = "Copyright (C) 2015, Nginx Inc. All rights reserved."
__credits__ = ["Mike Belov", "Andrei Belov", "Ivan Poluyanov", "Oleg Mamontov", "Andrew Alexeev"]
__license__ = ""
__maintainer__ = "Mike Belov"
__email__ = "dedm@nginx.com"


class NginxConfigCollector(AbstractCollector):

    short_name = 'nginx_config'

    def __init__(self, filename=None, upload=False, **kwargs):
        super(NginxConfigCollector, self).__init__(**kwargs)

        self.filename = filename
        self.have_to_upload = upload
        self.previous_mtimes = {}
        self.previous_checksum = None
        self.client = HTTPClient()

    def collect(self):
        try:
            config = NginxConfig(self.filename, binary=self.object.bin_path)

            for error in config.parser_errors:
                self.eventd.event(level=WARNING, message=error)
            for error in config.test_errors:
                self.eventd.event(level=CRITICAL, message=error)

            files_mtimes = {}
            for file_name, file_info in config.files.iteritems():
                files_mtimes[file_name] = file_info['mtime']
            if files_mtimes != self.previous_mtimes:
                checksum = config.checksum()
                if self.previous_checksum != checksum:
                    if self.have_to_upload:
                        self.upload(config, checksum)
                    self.previous_checksum = checksum
                self.previous_mtimes = files_mtimes
        except Exception, e:
            exception_name = e.__class__.__name__
            context.log.error('failed to collect due to %s' % exception_name)
            context.log.debug('additional info:', exc_info=True)

    def upload(self, config, checksum):
        payload = {
            'root': config.filename,
            'index': config.index,
            'tree': config.tree,
            'files': config.files,
            'errors': {'parser': len(config.parser_errors), 'test': len(config.test_errors)}
        }
        self.configd.config(payload=payload, checksum=checksum)
